import json
import random
import string

from django.conf import settings
from django.contrib.auth.models import User
from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _

from rest_framework import renderers, status
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.compat import coreapi, coreschema
from rest_framework.permissions import IsAuthenticated, AllowAny
from rest_framework.response import Response
from rest_framework.schemas import ManualSchema
from rest_framework.views import APIView, exception_handler

from slack import WebClient
from slack.errors import SlackApiError

from .tasks import send_mail_task


class ObtainCustomAuthToken(ObtainAuthToken):
    if coreapi is not None and coreschema is not None:
        schema = ManualSchema(
            fields=[
                coreapi.Field(
                    name="username",
                    required=True,
                    location='form',
                    schema=coreschema.String(
                        title="Username",
                        description="Valid username for authentication",
                    ),
                ),
                coreapi.Field(
                    name="password",
                    required=True,
                    location='form',
                    schema=coreschema.String(
                        title="Password",
                        description="Valid password for authentication",
                    ),
                ),
            ],
            description="""{RETRIEVE}""".format(
                RETRIEVE=_("Service to log in using token, with username and password"),
            ),
            encoding="application/json",
        )


ObtainCustomAuthToken.__doc__ = """{RETRIEVE}
""".format(
    RETRIEVE=_("Service to log in using token, with username and password"),
)


class LogoutView(APIView):
    throttle_classes = ()
    permission_classes = (IsAuthenticated,)
    renderer_classes = (renderers.JSONRenderer,)

    def post(self, request, *args, **kwargs):
        request.user.auth_token.delete()
        response_data = {'message': _('has closed session correctly')}
        return Response(response_data, status=status.HTTP_200_OK,
                        content_type="application/json")


LogoutView.__doc__ = """{RETRIEVE}""".format(
    RETRIEVE=_("Service to log out."),
)

class ForgetPasswordView(APIView):
    permission_classes = (AllowAny,)
    renderer_classes = (renderers.JSONRenderer,)
    if coreapi is not None and coreschema is not None:
        schema = ManualSchema(
            fields=[
                coreapi.Field(
                    name="username",
                    required=True,
                    location='form',
                    schema=coreschema.String(
                        title="Username",
                        description="Valid username for authentication",
                    ),
                ),
            ],
            description="""{RETRIEVE}""".format(
                RETRIEVE=_('Service to forget password. ** {"username": "user1"} **'),
            ),
            encoding="application/json",
        )

    def post(self, request, *args, **kwargs):
        username = request.data.get('username')
        user_detail = User.objects.filter(username=username).first()
        if user_detail:
            new_password = ''.join(
                random.choices(string.ascii_letters + string.digits, k=8))
            first = None
            user_detail.set_password(new_password)
            user_detail.save()
            if user_detail.first_name:
                if " " in user_detail.first_name:
                    first = (user_detail.first_name).split(" ")[0]
                else:
                    first = user_detail.first_name
            if first:
                message = "{FIRST} {MESSAGE} {NEW}".format(
                    MESSAGE=_("Your new password is:"),
                    NEW=_(new_password),
                    FIRST=first
                )
            else:
                message = "{MESSAGE} {NEW}".format(
                    MESSAGE=_("Your new password is:"),
                    NEW=_(new_password)
                )
            try:
                user_detail.auth_token.delete()
            except:
                pass
            send_mail_task.delay(
                email=user_detail.email,
                subject= _("Your new password is:"),
                message=message
            )
            return Response({"message": _('the new password has been send to Email.')},
                            status=status.HTTP_201_CREATED)
        else:
            return Response({"message": _('the new password has been send to Email.')},
                            status=status.HTTP_202_ACCEPTED)

ForgetPasswordView.__doc__ = """{RETRIEVE}""".format(
    RETRIEVE=_('Service to forget password. ** {"username": "user1"} **'),
)

def handler500(exception, context):
    response = exception_handler(exception, context)
    try:
        detail = response.data['detail']
    except:
        try:
            detail = exception.message
        except:
            try:
                detail = response.data
            except:
                detail = str(exception).replace("`", "'")
    try:
        status_code = response.status_code
    except:
        status_code = 500
    response = HttpResponse(
        json.dumps({'message': detail}),
        content_type="application/json",
        status=status_code
    )
    return response


def members_view(request):
    slack_token = settings.BOT_USER_ACCESS_TOKEN
    client = WebClient(token=slack_token)
    try:
        listclient = client.users_list()
        users = listclient["members"]
    except SlackApiError as e:
        users = e.response
    return HttpResponse(users,status=200)


from django.db.models import  CharField, BooleanField
from django.utils.translation import ugettext_lazy as _

from loducode_utils.models.audit import Audit


class City(Audit):
    STATES = (
        ('', '---------'),
        ('Amazonas', 'Amazonas'),
        ('Antioquia', 'Antioquia'),
        ('Arauca', 'Arauca'),
        ('Atlántico', 'Atlántico'),
        ('Bolívar', 'Bolívar'),
        ('Boyacá', 'Boyacá'),
        ('Caldas', 'Caldas'),
        ('Caquetá', 'Caquetá'),
        ('Casanare', 'Casanare'),
        ('Cauca', 'Cauca'),
        ('Cesar', 'Cesar'),
        ('Chocó', 'Chocó'),
        ('Córdoba', 'Córdoba'),
        ('Cundinamarca', 'Cundinamarca'),
        ('Guainía', 'Guainía'),
        ('Guaviare', 'Guaviare'),
        ('Huila', 'Huila'),
        ('La Guajira', 'La Guajira'),
        ('Magdalena', 'Magdalena'),
        ('Meta', 'Meta'),
        ('Nariño', 'Nariño'),
        ('Norte de Santander', 'Norte de Santander'),
        ('Putumayo', 'Putumayo'),
        ('Quindío', 'Quindío'),
        ('Risaralda', 'Risaralda'),
        ('San Andrés y Providencia', 'San Andrés y Providencia'),
        ('Santander', 'Santander'),
        ('Sucre', 'Sucre'),
        ('Tolima', 'Tolima'),
        ('Valle del Cauca', 'Valle del Cauca'),
        ('Vaupés', 'Vaupés'),
        ('Vichada', 'Vichada'),
        ('Bogotá d C.', 'Bogotá d C.'),
    )
    name: str = CharField(verbose_name=_('name'), max_length=100)
    state: str = CharField(verbose_name=_('departament'), max_length=100, choices=STATES)
    active: bool = BooleanField(verbose_name=_('active'),default=True)

    class Meta:
        verbose_name = _('city')
        verbose_name_plural = _('cities')

    def __str__(self):
        return f'{self.name} - {self.state}'


# Generated by Django 2.2 on 2021-03-18 15:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('loducode_utils', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='city',
            name='created_by',
            field=models.ForeignKey(blank=True, help_text='user who created the object', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='city_created_by', to=settings.AUTH_USER_MODEL, verbose_name='creation user'),
        ),
        migrations.AddField(
            model_name='city',
            name='modified_by',
            field=models.ForeignKey(blank=True, help_text='user who performed the update', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='city_modified_by', to=settings.AUTH_USER_MODEL, verbose_name='update user'),
        ),
    ]



from celery import shared_task
from django.core.mail import EmailMessage

from .utils import slack_send_message


@shared_task()
def send_mail_task(email, subject, message):
    msg = EmailMessage(subject=subject, body=message, bcc=[email])
    msg.content_subtype = "html"
    return msg.send()


@shared_task()
def send_message_slack_task(channel: str, message: str, id_user: str = ''):
    '''
    send messages slack
    :param channel: channel for send
    :param message: message send
    :param id_user: id user slack
    :return: (str) response
    '''
    res = slack_send_message(channel,message,id_user)
    return res


# Generated by Django 2.2 on 2021-03-18 15:05

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='City',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True, help_text='date when the object was created', verbose_name='creation date')),
                ('modified_at', models.DateTimeField(auto_now=True, help_text='date when the object was modified', verbose_name='update date')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
                ('state', models.CharField(choices=[('', '---------'), ('Amazonas', 'Amazonas'), ('Antioquia', 'Antioquia'), ('Arauca', 'Arauca'), ('Atlántico', 'Atlántico'), ('Bolívar', 'Bolívar'), ('Boyacá', 'Boyacá'), ('Caldas', 'Caldas'), ('Caquetá', 'Caquetá'), ('Casanare', 'Casanare'), ('Cauca', 'Cauca'), ('Cesar', 'Cesar'), ('Chocó', 'Chocó'), ('Córdoba', 'Córdoba'), ('Cundinamarca', 'Cundinamarca'), ('Guainía', 'Guainía'), ('Guaviare', 'Guaviare'), ('Huila', 'Huila'), ('La Guajira', 'La Guajira'), ('Magdalena', 'Magdalena'), ('Meta', 'Meta'), ('Nariño', 'Nariño'), ('Norte de Santander', 'Norte de Santander'), ('Putumayo', 'Putumayo'), ('Quindío', 'Quindío'), ('Risaralda', 'Risaralda'), ('San Andrés y Providencia', 'San Andrés y Providencia'), ('Santander', 'Santander'), ('Sucre', 'Sucre'), ('Tolima', 'Tolima'), ('Valle del Cauca', 'Valle del Cauca'), ('Vaupés', 'Vaupés'), ('Vichada', 'Vichada'), ('Bogotá d C.', 'Bogotá d C.')], max_length=100, verbose_name='departament')),
                ('active', models.BooleanField(default=True, verbose_name='active')),
            ],
            options={
                'verbose_name': 'city',
                'verbose_name_plural': 'cities',
            },
        ),
    ]



from django.contrib import admin


from import_export.admin import ImportExportModelAdmin
from loducode_utils.models.city import City


class AuditAdmin(admin.ModelAdmin):
    __readonly_audit_fields = (
        'created_at', 'modified_at', 'created_by', 'modified_by'
    )

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super(AuditAdmin, self).get_readonly_fields(request, obj)
        readonly_fields = readonly_fields + self.__readonly_audit_fields
        return readonly_fields

class AuditStackedInline(admin.StackedInline):
    __readonly_audit_fields = (
        'created_at', 'modified_at', 'created_by', 'modified_by'
    )

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super(AuditStackedInline, self).get_readonly_fields(request, obj)
        readonly_fields = readonly_fields + self.__readonly_audit_fields
        return readonly_fields

class AuditTabularInline(admin.TabularInline):
    __readonly_audit_fields = (
        'created_at', 'modified_at', 'created_by', 'modified_by'
    )

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super(AuditTabularInline, self).get_readonly_fields(request, obj)
        readonly_fields = readonly_fields + self.__readonly_audit_fields
        return readonly_fields

class ReadOnlyAdmin(AuditAdmin):
    pass

class ReadOnlyStackedInline(AuditStackedInline):
    pass

class ReadOnlyTabularInline(AuditTabularInline):
    pass


class CityAdmin(ImportExportModelAdmin ,AuditAdmin):
    list_display = ("name","state")
    list_display_links = ("name","state")
    list_filter = ("state",)
    search_fields = ("name",)

admin.site.register(City, CityAdmin)


from django.db.models import CharField

from loducode_utils.models.audit import Audit


class PaymentRecord(Audit):
    x_cust_id_cliente = CharField(max_length=250, blank=True, null=True)
    x_description = CharField(max_length=250, blank=True, null=True)
    x_amount_ok = CharField(max_length=250, blank=True, null=True)
    x_id_invoice = CharField(max_length=250, blank=True, null=True)
    x_amount_base = CharField(max_length=250, blank=True, null=True)
    x_tax = CharField(max_length=250, blank=True, null=True)
    x_currency_code = CharField(max_length=250, blank=True, null=True)
    x_franchise = CharField(max_length=250, blank=True, null=True)
    x_transaction_date = CharField(max_length=250, blank=True,
                                          null=True)
    x_approval_code = CharField(max_length=250, blank=True, null=True)
    x_transaction_id = CharField(max_length=250, blank=True, null=True)
    x_ref_payco = CharField(max_length=250, blank=True, null=True)
    x_cod_response = CharField(max_length=250, blank=True, null=True)
    x_cod_transaction_state = CharField(max_length=250, blank=True,
                                               null=True)
    x_transaction_state = CharField(max_length=250, blank=True,
                                           null=True)
    x_signature = CharField(max_length=250, blank=True, null=True)
    x_response = CharField(max_length=250, blank=True, null=True)
    x_response_reason_text = CharField(max_length=250, blank=True,
                                              null=True)
    x_extra1 = CharField(max_length=250, blank=True, null=True)
    x_extra2 = CharField(max_length=250, blank=True, null=True)
    x_extra3 = CharField(max_length=250, blank=True, null=True)
    x_amount = CharField(max_length=250, blank=True, null=True)
    x_amount_country = CharField(max_length=250, blank=True, null=True)
    x_bank_name = CharField(max_length=250, blank=True, null=True)
    x_cardnumber = CharField(max_length=250, blank=True, null=True)
    x_quotas = CharField(max_length=250, blank=True, null=True)
    x_fecha_transaccion = CharField(max_length=250, blank=True,
                                           null=True)
    x_errorcode = CharField(max_length=250, blank=True, null=True)
    x_customer_doctype = CharField(max_length=250, blank=True,
                                          null=True)
    x_customer_lastname = CharField(max_length=250, blank=True,
                                           null=True)
    x_customer_name = CharField(max_length=250, blank=True, null=True)
    x_customer_email = CharField(max_length=250, blank=True, null=True)
    x_customer_phone = CharField(max_length=250, blank=True, null=True)
    x_customer_country = CharField(max_length=250, blank=True,
                                          null=True)
    x_customer_city = CharField(max_length=250, blank=True, null=True)
    x_customer_address = CharField(max_length=250, blank=True,
                                          null=True)
    x_customer_ip = CharField(max_length=250, blank=True, null=True)
    x_test_request = CharField(max_length=250, blank=True, null=True)

    class Meta:
        abstract = True

    def __str__(self):
        return '%s'%self.id


from django.urls import path

from .views_api import ObtainCustomAuthToken, LogoutView, ForgetPasswordView

app_name = 'loducode_utils'

urlpatterns = [
    path('token/', ObtainCustomAuthToken.as_view(), name="token"),
    path('logout/', LogoutView.as_view(), name="logout"),
    path('forget/', ForgetPasswordView.as_view(), name="logout")
]



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='loducode_utils',
    packages=['loducode_utils','loducode_utils.migrations'],  # this must be the same as the name above
    version='1.3.3',
    description='Basic components for the development of loducode s.a.s.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Cristian Dulcey',
    author_email='cristian@loducode.com',
    url='https://github.com/UnTalDulcey/loducode_utils',  # use the URL to the github repo
    download_url='https://github.com/UnTalDulcey/loducode_utils/tarball/1.3.3',
    keywords=['utils', 'loducode_utils', 'loducode'],
    classifiers=[],
    install_requires=[i.strip() for i in open("requirements.txt").readlines()],
    setup_requires=['wheel'],
    include_package_data=True,
)



import json

from django.conf import settings
from django.views.generic import ListView

from slack import WebClient
from slack.errors import SlackApiError


class PaginatedListView(ListView):

    def pager_v2(self, rango=2):
        fin = int(self.object_list.__len__() / self.paginate_by)
        if self.object_list.__len__() / self.paginate_by > fin:
            fin += 1
        ini = 1
        page = self.request.GET.get('page', "1")
        if page.isdigit():
            page = int(page)
            ini = page - rango
            if ini <= 0:
                rango -= -1 + ini
                ini = 1
            if page + rango <= fin:
                fin = page + rango
            else:
                rango = (rango + page) - fin
                if ini - rango <= 0:
                    ini = 1
                else:
                    ini -= rango
        return range(ini, fin + 1)

    def get_context_data(self, **kwargs):
        context = super(PaginatedListView, self).get_context_data(**kwargs)
        page = self.request.GET.get("page", 1)
        context["current_page"] = int(page)
        pages = self.pager_v2(2)
        context["pages"] = pages
        return context


def confirmation_pay(model, request):
    data = request.POST
    id = data.get("x_extra1")
    if request.method == 'POST' and id:
        recharge = model.objects.filter(id=id).first()
        if recharge:
            recharge.x_cust_id_cliente = data.get("x_cust_id_cliente")
            recharge.x_description = data.get("x_description")
            recharge.x_amount_ok = data.get("x_amount_ok")
            recharge.x_id_invoice = data.get("x_id_invoice")
            recharge.x_amount_base = data.get("x_amount_base")
            recharge.x_tax = data.get("x_tax")
            recharge.x_currency_code = data.get("x_currency_code")
            recharge.x_franchise = data.get("x_franchise")
            recharge.x_transaction_date = data.get("x_transaction_date")
            recharge.x_approval_code = data.get("x_approval_code")
            recharge.x_transaction_id = data.get("x_transaction_id")
            recharge.x_ref_payco = data.get("x_ref_payco")
            recharge.x_cod_response = data.get("x_cod_response")
            recharge.x_cod_transaction_state = data.get("x_cod_transaction_state")
            recharge.x_transaction_state = data.get("x_transaction_state")
            recharge.x_signature = data.get("x_signature")
            recharge.x_response = data.get("x_response")
            recharge.x_response_reason_text = data.get("x_response_reason_text")
            recharge.x_extra1 = data.get("x_extra1")
            recharge.x_extra2 = data.get("x_extra2")
            recharge.x_extra3 = data.get("x_extra3")
            recharge.x_amount = data.get("x_amount")
            recharge.x_amount_country = data.get("x_amount_country")
            recharge.x_bank_name = data.get("x_bank_name")
            recharge.x_cardnumber = data.get("x_cardnumber")
            recharge.x_quotas = data.get("x_quotas")
            recharge.x_fecha_transaccion = data.get("x_fecha_transaccion")
            recharge.x_errorcode = data.get("x_errorcode")
            recharge.x_customer_doctype = data.get("x_customer_doctype")
            recharge.x_customer_lastname = data.get("x_customer_lastname")
            recharge.x_customer_name = data.get("x_customer_name")
            recharge.x_customer_email = data.get("x_customer_email")
            recharge.x_customer_phone = data.get("x_customer_phone")
            recharge.x_customer_country = data.get("x_customer_country")
            recharge.x_customer_city = data.get("x_customer_city")
            recharge.x_customer_address = data.get("x_customer_address")
            recharge.x_customer_ip = data.get("x_customer_ip")
            recharge.x_test_request = data.get("x_test_request")
            recharge.save()
            return recharge
        else:
            return None
    else:
        return None


def slack_send_message(channel: str, message: str, id_user: str = '') -> str:
    '''
    send messages slack
    :param channel: channel for send
    :param message: message send
    :param id_user: id user slack
    :return: (str) response
    '''
    slack_token = settings.BOT_USER_ACCESS_TOKEN
    client = WebClient(token=slack_token)
    try:
        if id_user != '':
            res = client.chat_postEphemeral(
                channel=channel,
                text=message,
                user=id_user
            )
        else:
            res = client.chat_postMessage(
                channel=channel,
                text=message
            )
        response = json.dumps(res.data, indent=4)
    except SlackApiError as e:
        response = e.response["error"]
        assert e.response["error"]
    return response



import uuid

from crum import get_current_user
from django.contrib.auth import get_user_model
from django.db.models import Model, UUIDField, DateTimeField, ForeignKey, CASCADE
from django.utils.translation import ugettext_lazy as _


class Audit(Model):
    '''Audit Model
    AuditModel acts as an abstract base class from which every
    other model in the project will inherit. This class provides
    every table with the following attributes:
        + created_at (DateTime): Stores the datetime the object was created.
        + modified_at (DateTime): Stores the last datetime the object was modified.
        + created_by (ForeignKey): Stores the user who created the object.
        + modified_by (ForeignKey): Stores the user who modified the object.
    '''
    id: UUIDField = UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    created_at = DateTimeField(
        auto_now_add=True,
        verbose_name=_('creation date'),
        help_text=_('date when the object was created'),
        db_index=True
    )
    modified_at = DateTimeField(
        auto_now=True,
        verbose_name=_('update date'),
        help_text=_('date when the object was modified'),
    )
    created_by = ForeignKey(
        get_user_model(), on_delete=CASCADE,
        related_name='%(class)s_created_by',
        null=True, blank=True,
        verbose_name=_('creation user'),
        help_text=_('user who created the object'),
    )
    modified_by = ForeignKey(
        get_user_model(), on_delete=CASCADE,
        related_name='%(class)s_modified_by',
        null=True, blank=True,
        verbose_name=_('update user'),
        help_text=_('user who performed the update'),
    )

    class Meta:
        abstract = True
        ordering = ['-created_at']

    def save(self, *args, **kwargs):
        user = get_current_user()
        if user:
            if self.created_at is None and not user.is_anonymous:
                self.created_by = user
                self.modified_by = user
            elif not user.is_anonymous:
                self.modified_by = user
        super(Audit, self).save(*args, **kwargs)